function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { getItemsCount, getNextHighlightedIndex } from './utils';
export var stateReducer = function stateReducer(state, action) {
  switch (action.type) {
    case 'setHighlightedIndex':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          highlightedIndex: action.payload
        });
      }

    case 'setQuery':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          query: action.payload
        });
      }

    case 'setSuggestions':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          suggestions: action.payload
        });
      }

    case 'setIsOpen':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          isOpen: action.payload
        });
      }

    case 'setStatus':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          status: action.payload
        });
      }

    case 'setContext':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          context: _objectSpread(_objectSpread({}, state.context), action.payload)
        });
      }

    case 'ArrowDown':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          highlightedIndex: getNextHighlightedIndex(1, state.highlightedIndex, getItemsCount(state), action.props.defaultHighlightedIndex)
        });
      }

    case 'ArrowUp':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          highlightedIndex: getNextHighlightedIndex(-1, state.highlightedIndex, getItemsCount(state), action.props.defaultHighlightedIndex)
        });
      }

    case 'Escape':
      {
        if (state.isOpen) {
          return _objectSpread(_objectSpread({}, state), {}, {
            isOpen: false
          });
        }

        return _objectSpread(_objectSpread({}, state), {}, {
          query: '',
          status: 'idle',
          statusContext: {},
          suggestions: []
        });
      }

    case 'submit':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          highlightedIndex: null,
          isOpen: false,
          status: 'idle',
          statusContext: {}
        });
      }

    case 'reset':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          highlightedIndex: // Since we open the menu on reset when openOnFocus=true
          // we need to restore the highlighted index to the defaultHighlightedIndex. (DocSearch use-case)
          // Since we close the menu when openOnFocus=false
          // we lose track of the highlighted index. (Query-suggestions use-case)
          action.props.openOnFocus === true ? action.props.defaultHighlightedIndex : null,
          isOpen: action.props.openOnFocus,
          // @TODO: Check with UX team if we want to close the menu on reset.
          status: 'idle',
          statusContext: {},
          query: ''
        });
      }

    case 'focus':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          highlightedIndex: action.props.defaultHighlightedIndex,
          isOpen: action.props.openOnFocus || state.query.length > 0
        });
      }

    case 'blur':
      {
        if (action.props.debug) {
          return state;
        }

        return _objectSpread(_objectSpread({}, state), {}, {
          isOpen: false,
          highlightedIndex: null
        });
      }

    case 'mousemove':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          highlightedIndex: action.payload
        });
      }

    case 'mouseleave':
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          highlightedIndex: action.props.defaultHighlightedIndex
        });
      }

    default:
      return state;
  }
};